@@ -151,7 +151,7 @@ You can provide multiple lookup keys to this command, but it only returns a
 value for the first found key, omitting the rest.
 
 For more details about how Hiera works, see the Hiera documentation:
-https://docs.puppet.com/puppet/latest/hiera_intro.html
+https://puppet.com/docs/puppet/latest/hiera_intro.html
 
 OPTIONS
 -------
@@ -353,7 +353,7 @@ Copyright (c) 2015 Puppet Inc., LLC Licensed under the Apache 2.0 License
       if fact_file.end_with?("json")
         given_facts = Puppet::Util::Json.load(Puppet::FileSystem.read(fact_file, :encoding => 'utf-8'))
       else
-        given_facts = YAML.load(Puppet::FileSystem.read(fact_file, :encoding => 'utf-8'))
+        given_facts = Puppet::Util::Yaml.safe_load_file(fact_file)
       end
 
       unless given_facts.instance_of?(Hash)
