@@ -84,46 +84,6 @@ class Puppet::Settings
     "puppet.conf"
   end
 
-  def stringify_settings(section, settings = :all)
-    values_from_the_selected_section =
-      values(nil, section.to_sym)
-
-    loader_settings = {
-      :environmentpath => values_from_the_selected_section.interpolate(:environmentpath),
-      :basemodulepath => values_from_the_selected_section.interpolate(:basemodulepath),
-    }
-
-    Puppet.override(Puppet.base_context(loader_settings),
-                    _("New environment loaders generated from the requested section.")) do
-      # And now we can lookup values that include those from environments configured from
-      # the requested section
-      values = values(Puppet[:environment].to_sym, section.to_sym)
-
-      to_be_rendered = {}
-      settings = Puppet.settings.to_a.collect(&:first) if settings == :all
-      settings.sort.each do |setting_name|
-        to_be_rendered[setting_name] = values.print(setting_name.to_sym)
-      end
-
-      stringifyhash(to_be_rendered)
-    end
-  end
-
-  def stringifyhash(hash)
-    newhash = {}
-    hash.each do |key, val|
-      key = key.to_s
-      if val.is_a? Hash
-        newhash[key] = stringifyhash(val)
-      elsif val.is_a? Symbol
-        newhash[key] = val.to_s
-      else
-        newhash[key] = val
-      end
-    end
-    newhash
-  end
-
   # Create a new collection of config settings.
   def initialize
     @config = {}
@@ -295,7 +255,7 @@ class Puppet::Settings
     @global_defaults_initialized
   end
 
-  def initialize_global_settings(args = [], require_config = true)
+  def initialize_global_settings(args = [])
     raise Puppet::DevError, _("Attempting to initialize global default settings more than once!") if global_defaults_initialized?
 
     # The first two phases of the lifecycle of a puppet application are:
@@ -304,7 +264,7 @@ class Puppet::Settings
     # 2) Parse the puppet config file(s).
 
     parse_global_options(args)
-    parse_config_files(require_config)
+    parse_config_files
 
     @global_defaults_initialized = true
   end
@@ -630,19 +590,14 @@ class Puppet::Settings
   end
 
   # Parse the configuration file.  Just provides thread safety.
-  def parse_config_files(require_config = true)
+  def parse_config_files
     file = which_configuration_file
     if Puppet::FileSystem.exist?(file)
       begin
         text = read_file(file)
       rescue => detail
-        message = _("Could not load %{file}: %{detail}") % { file: file, detail: detail}
-        if require_config
-          Puppet.log_and_raise(detail, message)
-        else
-          Puppet.log_exception(detail, message)
-          return
-        end
+        Puppet.log_exception(detail, "Could not load #{file}: #{detail}")
+        return
       end
     else
       return
@@ -1229,10 +1184,10 @@ Generated on #{Time.now}.
       if !Puppet::FileSystem.symlink?(configured_environment_path)
         parameters = { :ensure => 'directory' }
         unless Puppet::FileSystem.exist?(configured_environment_path)
-          parameters[:mode] = '0750'
+          parameters.merge!(:mode => '0750')
           if Puppet.features.root?
-            parameters[:owner] = Puppet[:user] if service_user_available?
-            parameters[:group] = Puppet[:group] if service_group_available?
+            parameters.merge!(:owner => Puppet[:user]) if service_user_available?
+            parameters.merge!(:group => Puppet[:group]) if service_group_available?
           end
         end
         catalog.add_resource(Puppet::Resource.new(:file, configured_environment_path, :parameters => parameters))
@@ -1242,7 +1197,7 @@ Generated on #{Time.now}.
 
   def add_user_resources(catalog, sections)
     return unless Puppet.features.root?
-    return if Puppet.features.microsoft_windows?
+    return if Puppet::Util::Platform.windows?
     return unless self[:mkusers]
 
     @config.each do |name, setting|
