@@ -1,4 +1,3 @@
-# coding: utf-8
 require 'puppet/util/windows'
 require 'ffi'
 
@@ -181,30 +180,7 @@ module Puppet::Util::Windows
     # // Value to indicate no change to an optional parameter
     # //
     # #define SERVICE_NO_CHANGE              0xffffffff
-    # https://docs.microsoft.com/en-us/windows/win32/api/winsvc/nf-winsvc-changeserviceconfig2w
-    SERVICE_CONFIG_DESCRIPTION              = 0x00000001
-    SERVICE_CONFIG_FAILURE_ACTIONS          = 0x00000002
-    SERVICE_CONFIG_DELAYED_AUTO_START_INFO  = 0x00000003
-    SERVICE_CONFIG_FAILURE_ACTIONS_FLAG     = 0x00000004
-    SERVICE_CONFIG_SERVICE_SID_INFO         = 0x00000005
-    SERVICE_CONFIG_REQUIRED_PRIVILEGES_INFO = 0x00000006
-    SERVICE_CONFIG_PRESHUTDOWN_INFO         = 0x00000007
-    SERVICE_CONFIG_TRIGGER_INFO             = 0x00000008
-    SERVICE_CONFIG_PREFERRED_NODE           = 0x00000009
-    SERVICE_CONFIG_LAUNCH_PROTECTED         = 0x00000012
-    SERVICE_NO_CHANGE                       = 0xffffffff
-    SERVICE_CONFIG_TYPES = {
-      SERVICE_CONFIG_DESCRIPTION => :SERVICE_CONFIG_DESCRIPTION,
-      SERVICE_CONFIG_FAILURE_ACTIONS => :SERVICE_CONFIG_FAILURE_ACTIONS,
-      SERVICE_CONFIG_DELAYED_AUTO_START_INFO => :SERVICE_CONFIG_DELAYED_AUTO_START_INFO,
-      SERVICE_CONFIG_FAILURE_ACTIONS_FLAG => :SERVICE_CONFIG_FAILURE_ACTIONS_FLAG,
-      SERVICE_CONFIG_SERVICE_SID_INFO => :SERVICE_CONFIG_SERVICE_SID_INFO,
-      SERVICE_CONFIG_REQUIRED_PRIVILEGES_INFO => :SERVICE_CONFIG_REQUIRED_PRIVILEGES_INFO,
-      SERVICE_CONFIG_PRESHUTDOWN_INFO => :SERVICE_CONFIG_PRESHUTDOWN_INFO,
-      SERVICE_CONFIG_TRIGGER_INFO => :SERVICE_CONFIG_TRIGGER_INFO,
-      SERVICE_CONFIG_PREFERRED_NODE => :SERVICE_CONFIG_PREFERRED_NODE,
-      SERVICE_CONFIG_LAUNCH_PROTECTED => :SERVICE_CONFIG_LAUNCH_PROTECTED,
-    }
+    SERVICE_NO_CHANGE = 0xffffffff
 
     # Service enum codes
     # https://docs.microsoft.com/en-us/windows/desktop/api/winsvc/nf-winsvc-enumservicesstatusexa
@@ -243,19 +219,6 @@ module Puppet::Util::Windows
       )
     end
 
-    # https://docs.microsoft.com/en-us/windows/win32/api/winsvc/ns-winsvc-service_delayed_auto_start_info
-    # typedef struct _SERVICE_DELAYED_AUTO_START_INFO {
-    #   BOOL fDelayedAutostart;
-    # } SERVICE_DELAYED_AUTO_START_INFO, *LPSERVICE_DELAYED_AUTO_START_INFO;
-    class SERVICE_DELAYED_AUTO_START_INFO < FFI::Struct
-      layout(:fDelayedAutostart, :int)
-      alias aset []=
-      # Intercept the accessor so that we can handle either true/false or 1/0.
-      # Since there is only one member, there’s no need to check the key name.
-      def []=(key, value)
-        [0, false].include?(value) ? aset(key, 0) : aset(key, 1)
-      end
-    end
 
     # https://docs.microsoft.com/en-us/windows/desktop/api/winsvc/ns-winsvc-_enum_service_status_processw
     # typedef struct _ENUM_SERVICE_STATUS_PROCESSW {
@@ -319,7 +282,7 @@ module Puppet::Util::Windows
 
     # Returns true if the service exists, false otherwise.
     #
-    # @param [:string] service_name name of the service
+    # @param [String] service_name name of the service
     def exists?(service_name)
       open_service(service_name, SC_MANAGER_CONNECT, SERVICE_QUERY_STATUS) do |_|
         true
@@ -332,9 +295,10 @@ module Puppet::Util::Windows
 
     # Start a windows service
     #
-    # @param [:string] service_name name of the service to start
-    def start(service_name)
-      Puppet.debug _("Starting the %{service_name} service") % { service_name: service_name }
+    # @param [String] service_name name of the service to start
+    # @param optional [Integer] timeout the minumum number of seconds to wait before timing out
+    def start(service_name, timeout: DEFAULT_TIMEOUT)
+      Puppet.debug _("Starting the %{service_name} service. Timeout set to: %{timeout} seconds") % { service_name: service_name, timeout: timeout }
 
       valid_initial_states = [
         SERVICE_STOP_PENDING,
@@ -342,7 +306,7 @@ module Puppet::Util::Windows
         SERVICE_START_PENDING
       ]
 
-      transition_service_state(service_name, valid_initial_states, SERVICE_RUNNING) do |service|
+      transition_service_state(service_name, valid_initial_states, SERVICE_RUNNING, timeout) do |service|
         if StartServiceW(service, 0, FFI::Pointer::NULL) == FFI::WIN32_FALSE
           raise Puppet::Util::Windows::Error, _("Failed to start the service")
         end
@@ -354,13 +318,14 @@ module Puppet::Util::Windows
 
     # Stop a windows service
     #
-    # @param [:string] service_name name of the service to stop
-    def stop(service_name)
-      Puppet.debug _("Stopping the %{service_name} service") % { service_name: service_name }
+    # @param [String] service_name name of the service to stop
+    # @param optional [Integer] timeout the minumum number of seconds to wait before timing out
+    def stop(service_name, timeout: DEFAULT_TIMEOUT)
+      Puppet.debug _("Stopping the %{service_name} service. Timeout set to: %{timeout} seconds") % { service_name: service_name, timeout: timeout }
 
       valid_initial_states = SERVICE_STATES.keys - [SERVICE_STOPPED]
 
-      transition_service_state(service_name, valid_initial_states, SERVICE_STOPPED) do |service|
+      transition_service_state(service_name, valid_initial_states, SERVICE_STOPPED, timeout) do |service|
         send_service_control_signal(service, SERVICE_CONTROL_STOP)
       end
 
@@ -370,9 +335,10 @@ module Puppet::Util::Windows
 
     # Resume a paused windows service
     #
-    # @param [:string] service_name name of the service to resume
-    def resume(service_name)
-      Puppet.debug _("Resuming the %{service_name} service") % { service_name: service_name }
+    # @param [String] service_name name of the service to resume
+    # @param optional [Integer] :timeout the minumum number of seconds to wait before timing out
+    def resume(service_name, timeout: DEFAULT_TIMEOUT)
+      Puppet.debug _("Resuming the %{service_name} service. Timeout set to: %{timeout} seconds") % { service_name: service_name, timeout: timeout }
 
       valid_initial_states = [
         SERVICE_PAUSE_PENDING,
@@ -380,10 +346,10 @@ module Puppet::Util::Windows
         SERVICE_CONTINUE_PENDING
       ]
 
-      transition_service_state(service_name, valid_initial_states, SERVICE_RUNNING) do |service|
+      transition_service_state(service_name, valid_initial_states, SERVICE_RUNNING, timeout) do |service|
         # The SERVICE_CONTROL_CONTINUE signal can only be sent when
         # the service is in the SERVICE_PAUSED state
-        wait_on_pending_state(service, SERVICE_PAUSE_PENDING)
+        wait_on_pending_state(service, SERVICE_PAUSE_PENDING, timeout)
 
         send_service_control_signal(service, SERVICE_CONTROL_CONTINUE)
       end
@@ -394,7 +360,7 @@ module Puppet::Util::Windows
 
     # Query the state of a service using QueryServiceStatusEx
     #
-    # @param [:string] service_name name of the service to query
+    # @param [string] service_name name of the service to query
     # @return [string] the status of the service
     def service_state(service_name)
       state = nil
@@ -411,9 +377,8 @@ module Puppet::Util::Windows
     module_function :service_state
 
     # Query the configuration of a service using QueryServiceConfigW
-    # or QueryServiceConfig2W
     #
-    # @param [:string] service_name name of the service to query
+    # @param [String] service_name name of the service to query
     # @return [QUERY_SERVICE_CONFIGW.struct] the configuration of the service
     def service_start_type(service_name)
       start_type = nil
@@ -422,14 +387,6 @@ module Puppet::Util::Windows
           start_type = SERVICE_START_TYPES[config[:dwStartType]]
         end
       end
-      # if the service has type AUTO_START, check if it's a delayed service
-      if start_type == :SERVICE_AUTO_START
-        open_service(service_name, SC_MANAGER_CONNECT, SERVICE_QUERY_CONFIG) do |service|
-          query_config2(service, SERVICE_CONFIG_DELAYED_AUTO_START_INFO) do |config|
-            return :SERVICE_DELAYED_AUTO_START if config[:fDelayedAutostart] == 1
-          end
-        end
-      end
       if start_type.nil?
         raise Puppet::Error.new(_("Unknown start type '%{start_type}' for '%{service_name}'") % { start_type: start_type.to_s, service_name: service_name})
       end
@@ -439,12 +396,11 @@ module Puppet::Util::Windows
 
     # Change the startup mode of a windows service
     #
-    # @param [String] service_name the name of the service to modify
-    # @param [Integer] startup_type a code corresponding to a start type for
+    # @param [string] service_name the name of the service to modify
+    # @param [Int] startup_type a code corresponding to a start type for
     #  windows service, see the "Service start type codes" section in the
     #  Puppet::Util::Windows::Service file for the list of available codes
-    # @param [Bool] delayed whether the service should be started with a delay
-    def set_startup_mode(service_name, startup_type, delayed=false)
+    def set_startup_mode(service_name, startup_type)
       startup_code = SERVICE_START_TYPES.key(startup_type)
       if startup_code.nil?
         raise Puppet::Error.new(_("Unknown start type %{start_type}") % {startup_type: startup_type.to_s})
@@ -471,7 +427,6 @@ module Puppet::Util::Windows
           raise Puppet::Util::Windows::Error.new(_("Failed to update service configuration"))
         end
       end
-      set_startup_mode_delayed(service_name, delayed)
     end
     module_function :set_startup_mode
 
@@ -608,8 +563,9 @@ module Puppet::Util::Windows
       #
       # @param [String] service_name the name of the service to transition
       # @param [[Integer]] valid_initial_states an array of valid states that the service can transition from
-      # @param [:Integer] final_state the state that the service will transition to
-      def transition_service_state(service_name, valid_initial_states, final_state, &block)
+      # @param [Integer] final_state the state that the service will transition to
+      # @param [Integer] timeout the minumum number of seconds to wait before timing out
+      def transition_service_state(service_name, valid_initial_states, final_state, timeout, &block)
         service_access = SERVICE_START | SERVICE_STOP | SERVICE_PAUSE_CONTINUE | SERVICE_QUERY_STATUS
         open_service(service_name, SC_MANAGER_CONNECT, service_access) do |service|
           query_status(service) do |status|
@@ -641,7 +597,7 @@ module Puppet::Util::Windows
             end
             if possible_pending_states.include?(initial_state)
               Puppet.debug _("There is already a pending transition to the %{final_state} state for the %{service_name} service.")  % { final_state: SERVICE_STATES[final_state], service_name: service_name }
-              wait_on_pending_state(service, initial_state)
+              wait_on_pending_state(service, initial_state, timeout)
 
               next
             end
@@ -657,7 +613,7 @@ module Puppet::Util::Windows
             # to perform the state transition (it may not even exist).
             if UNSAFE_PENDING_STATES.include?(initial_state)
               Puppet.debug _("The service is in the %{pending_state} state, which is an unsafe pending state.") % { pending_state: SERVICE_STATES[initial_state] }
-              wait_on_pending_state(service, initial_state)
+              wait_on_pending_state(service, initial_state, timeout)
               initial_state = FINAL_STATES[initial_state]
             end
 
@@ -666,7 +622,7 @@ module Puppet::Util::Windows
             yield service
 
             Puppet.debug _("Waiting for the transition to finish")
-            wait_on_state_transition(service, initial_state, final_state)
+            wait_on_state_transition(service, initial_state, final_state, timeout)
           end
         end
       rescue => detail
@@ -753,82 +709,6 @@ module Puppet::Util::Windows
       end
       private :query_config
 
-      # @api private
-      # perform QueryServiceConfig2W on a windows service and return the
-      # result
-      #
-      # @param [:handle] service handle of the service to query
-      # @param [Integer] info_level the configuration information to be queried
-      # @return [QUERY_SERVICE_CONFIG2W struct] the result of the query
-      def query_config2(service, info_level, &block)
-        config = nil
-        size_required = nil
-        # Fetch the bytes of memory required to be allocated
-        # for QueryServiceConfig2W to return succesfully. This
-        # is done by sending NULL and 0 for the pointer and size
-        # respectively, letting the command fail, then reading the
-        # value of pcbBytesNeeded
-        FFI::MemoryPointer.new(:lpword) do |bytes_pointer|
-          # return value will be false from this call, since it's designed
-          # to fail. Just ignore it
-          QueryServiceConfig2W(service, info_level, FFI::Pointer::NULL, 0, bytes_pointer)
-          size_required = bytes_pointer.read_dword
-          FFI::MemoryPointer.new(size_required) do |ssp_ptr|
-            # We need to supply the appropriate struct to be created based on
-            # the info_level
-            case info_level
-            when SERVICE_CONFIG_DELAYED_AUTO_START_INFO
-              config = SERVICE_DELAYED_AUTO_START_INFO.new(ssp_ptr)
-            end
-            success = QueryServiceConfig2W(
-              service,
-              info_level,
-              ssp_ptr,
-              size_required,
-              bytes_pointer
-            )
-            if success == FFI::WIN32_FALSE
-              raise Puppet::Util::Windows::Error.new(_("Service query for %{parameter_name} failed") % { parameter_name: SERVICE_CONFIG_TYPES[info_level] } )
-            end
-            yield config
-          end
-        end
-      end
-      private :query_config2
-
-      # @api private
-      # Sets an optional parameter on a service by calling
-      # ChangeServiceConfig2W
-      #
-      # @param [String] service_name name of service
-      # @param [Integer] change parameter to change
-      # @param [struct] value appropriate struct based on the parameter to change
-      def set_optional_parameter(service_name, change, value)
-        open_service(service_name, SC_MANAGER_CONNECT, SERVICE_CHANGE_CONFIG) do |service|
-          success = ChangeServiceConfig2W(
-            service,
-            change, # dwInfoLevel
-            value,  # lpInfo
-          )
-          if success == FFI::WIN32_FALSE
-            raise Puppet::Util::windows::Error.new(_("Failed to update service %{change} configuration") % { change: change } )
-          end
-        end
-      end
-      private :set_optional_parameter
-
-      # @api private
-      # Controls the delayed auto-start setting of a service
-      #
-      # @param [String] service_name name of service
-      # @param [Bool] delayed whether the service should be started with a delay or not
-      def set_startup_mode_delayed(service_name, delayed)
-        delayed_start = SERVICE_DELAYED_AUTO_START_INFO.new
-        delayed_start[:fDelayedAutostart] = delayed
-        set_optional_parameter(service_name, SERVICE_CONFIG_DELAYED_AUTO_START_INFO, delayed_start)
-      end
-      private :set_startup_mode_delayed
-
       # @api private
       # Sends a service control signal to a service
       #
@@ -850,7 +730,8 @@ module Puppet::Util::Windows
       # @param [:handle] service handle to the service to wait on
       # @param [Integer] initial_state the state that the service is transitioning from.
       # @param [Integer] final_state the state that the service is transitioning to
-      def wait_on_state_transition(service, initial_state, final_state)
+      # @param [Integer] timeout the minumum number of seconds to wait before timing out
+      def wait_on_state_transition(service, initial_state, final_state, timeout)
         # Get the pending state for this transition. Note that SERVICE_RUNNING
         # has two possible pending states, which is why we need this logic.
         if final_state != SERVICE_RUNNING
@@ -866,14 +747,14 @@ module Puppet::Util::Windows
         # Wait for the transition to finish
         state = nil
         elapsed_time = 0
-        while elapsed_time <= DEFAULT_TIMEOUT
+        while elapsed_time <= timeout
 
           query_status(service) do |status|
             state = status[:dwCurrentState]
             return if state == final_state
             if state == pending_state
               Puppet.debug _("The service transitioned to the %{pending_state} state.") % { pending_state: SERVICE_STATES[pending_state] }
-              wait_on_pending_state(service, pending_state)
+              wait_on_pending_state(service, pending_state, timeout)
               return
             end
             sleep(1)
@@ -897,7 +778,8 @@ module Puppet::Util::Windows
       #
       # @param [:handle] service handle to the service to wait on
       # @param [Integer] pending_state the pending state
-      def wait_on_pending_state(service, pending_state)
+      # @param [Integer] timeout the minumum number of seconds to wait before timing out
+      def wait_on_pending_state(service, pending_state, timeout)
         final_state = FINAL_STATES[pending_state]
 
         Puppet.debug _("Waiting for the pending transition to the %{final_state} state to finish.") % { final_state: SERVICE_STATES[final_state] }
@@ -924,8 +806,8 @@ module Puppet::Util::Windows
               elapsed_time = 0
               last_checkpoint = checkpoint
             else
-              wait_hint_in_seconds = milliseconds_to_seconds(wait_hint)
-              timeout = wait_hint_in_seconds < DEFAULT_TIMEOUT ? DEFAULT_TIMEOUT : wait_hint_in_seconds
+              wait_hint = milliseconds_to_seconds(status[:dwWaitHint])
+              timeout = wait_hint < timeout ? timeout : wait_hint
 
               if elapsed_time >= timeout
                 raise Puppet::Error, _("Timed out while waiting for the pending transition from %{pending_state} to %{final_state} to finish. The current state is %{current_state}.") % { pending_state: SERVICE_STATES[pending_state], final_state: SERVICE_STATES[final_state], current_state: SERVICE_STATES[state] }
@@ -1023,18 +905,6 @@ module Puppet::Util::Windows
     attach_function_private :QueryServiceConfigW,
       [:handle, :lpbyte, :dword, :lpdword], :win32_bool
 
-    # https://docs.microsoft.com/en-us/windows/win32/api/winsvc/nf-winsvc-queryserviceconfig2w
-    # BOOL QueryServiceConfig2W(
-    #   SC_HANDLE hService,
-    #   DWORD     dwInfoLevel,
-    #   LPBYTE    lpBuffer,
-    #   DWORD     cbBufSize,
-    #   LPDWORD   pcbBytesNeeded
-    # );
-    ffi_lib :advapi32
-    attach_function_private :QueryServiceConfig2W,
-      [:handle, :dword, :lpbyte, :dword, :lpdword], :win32_bool
-
     # https://docs.microsoft.com/en-us/windows/desktop/api/Winsvc/nf-winsvc-startservicew
     # BOOL StartServiceW(
     #   SC_HANDLE hService,
@@ -1085,15 +955,6 @@ module Puppet::Util::Windows
         :lpcwstr
       ], :win32_bool
 
-    # https://docs.microsoft.com/en-us/windows/win32/api/winsvc/nf-winsvc-changeserviceconfig2w
-    # BOOL ChangeServiceConfig2W(
-    #   SC_HANDLE hService,
-    #   DWORD     dwInfoLevel,
-    #   LPVOID    lpInfo
-    # );
-    ffi_lib :advapi32
-    attach_function_private :ChangeServiceConfig2W,
-      [:handle, :dword, :lpvoid], :win32_bool
 
     # https://docs.microsoft.com/en-us/windows/desktop/api/winsvc/nf-winsvc-enumservicesstatusexw
     # BOOL EnumServicesStatusExW(
